jQuery( function ( $ ) {
	var wpfEventTracking = {
		init: function(){
			this.optionPageRepeater();
			this.multiKeyRepeater();

			$(document).on('input change','.wpf-event-tracking input[type=text]',this.inputPreview);
			$(document).on('click','.wpf-event-tracking a.open-list', this.selectorShortcodes);
			$(document).on('click','.wpf-circles button[data-repeater-create]',this.repeaterButtonsFix);
			$(document).on('change','td.trigger select.trigger', this.select4Fix);
			$(document).on('click','.wpf-send-test',this.sendEventTest);

			// EDD Recurring Subscription.
			$(document).on('input change','#edd_recurring',this.eddRecurringFix);
			if($('#edd_recurring').length > 0){
				if($('#edd_recurring').val() === 'no'){
					$('.wpf-event-tracking').find('[data-trigger=edd_subscription_status_changed]').parents('table').hide();
				}
			}

			// Trigger preview on page load.
			$('.wpf-event-tracking input[type=text]').trigger('change');

		},

		/**
		 * Send an event test.
		 * @param {object} e 
		 */
		sendEventTest:function(e){
			e.preventDefault();
			// Change back text if it was sent.
			$(this).html('<span class="dashicons dashicons-bell wpf-ring"></span>Send Test');

			
			let trigger = $(this).parents('span.wpf-event-tracking');
			let data = {};
			data.integration_name = trigger.attr('data-integration');
			data.event_name = trigger.find('.wpf-preview .event-name').text();

			if(trigger.find('.wpf-multi-key-crm').length > 0){
				data.event_keys_values = [];
				$.each(trigger.find('.wpf-et-values tr'),function(index,item){
					var key = $(item).find('td:first').text().slice(0,-1);
					var value = $(item).find('td:last').text();
					
					data.event_keys_values[index] = {
						key: key,
						value: value
					};
				});
			
			}else{
				data.event_keys_values = trigger.find('.wpf-preview .event-value').text();
			}

			$.ajax({
				type: "POST",
				url: wpfEventData.ajaxurl,
				data: { action: 'wpf_et_send_test',data,'_ajax_nonce':wpfEventData.nonce },
				success: function(response){
					if(response.success === true){
						trigger.find('.wpf-send-test').text('Sent!');
					}
				},
				error: function(error){
					console.log(error);
				}
			});
	
		},

		/**
		 * Fix select4 selector when a trigger is added/changed.
		 */
		select4Fix:function(){
			$(this).closest( 'tr' ).find( 'select.select4-hidden-accessible option, select.select4-hidden-accessible optgroup' ).remove();
			$(this).closest( 'tr' ).find( 'select.select4-hidden-accessible' ).select4( 'destroy' ); // clear out the old options
			$(this).next( 'span.description' ).html( $(this).find( 'option:selected' ).attr( 'data-description' ) ); // update the description
			$(this).closest( 'tr' ).find( 'span.wpf-event-tracking' ).attr( 'data-trigger', $(this).find( 'option:selected' ).val() ); // update the data atts on the container.

		},

		/**
		 * Fix repeater button not clickable when it generates.
		 */
		repeaterButtonsFix:function(){
			if($(this).parents('.nr-item').is(':last-child')){
				$(this).parents('.wpf-multi-key-crm').find('.nr-item:first-child .wpf-circles button[data-repeater-create]').click();
			}
		},

		/**
		 * Fix table view in EDD recurring plugin.
		 */
		eddRecurringFix:function(){
			if($(this).val() === 'yes'){
				$('.wpf-event-tracking').find('[data-trigger=edd_subscription_status_changed]').parents('table').show();
			}else{
				$('.wpf-event-tracking').find('[data-trigger=edd_subscription_status_changed]').parents('table').hide();
			}
		},

		/**
		 * Multi key crm repeater.
		 */
		multiKeyRepeater:function(){
			if( $( '.wpf-event-tracking.single-trigger .wpf-multi-key-crm' ).length ){
				$( '.wpf-event-tracking.single-trigger .wpf-multi-key-crm' ).repeater({
					initEmpty: false,
					show: function () {
	
						$( this ).show();
	
						// Fix select4
						$( '.select4-container',$( this ) ).remove();
						$( '.trigger option:first',$( this ) ).attr( 'selected',false ).attr( 'selected',true );
						$( '.wpf-preview .event-name,.wpf-preview .event-value,.wpf-preview .event-key',$( this ) ).html( '' );
	
					},
	
					isFirstItemUndeletable: false
				});
			}
		},

		/**
		 * Tooltip charachters limit.
		 * @param {integer} limit 
		 */
		limitCharachters:function(limit){
			var title_limit_notice = wpfEventData.crms_notices.title_limit.replace('{limit}',limit);
	
			// Limit title.
			$('.wpf-event-tracking .wpf-et-name').each(function(index,this_field){
				this_field = $(this);

				if(this_field.val().length > limit){
					this_field.addClass('wpf-tip wpf-tip-bottom').attr('data-tip', title_limit_notice);
					$( '.wpf-tip.wpf-tip-bottom' ).tipTip({
						'attribute': 'data-tip',
						'delay': 0,
						'activation':'focus',
						'defaultPosition': 'top',
					});
					this_field.css('color','#cc0000');
				}
	
			});
		
			$(document).on('input','.wpf-event-tracking .wpf-et-name', function() {
				var this_field = $(this);

				if(this_field.val().length > limit){
					this_field.addClass('wpf-tip wpf-tip-bottom').attr('data-tip', title_limit_notice);
					$( '.wpf-tip.wpf-tip-bottom' ).tipTip({
						'attribute': 'data-tip',
						'delay': 0,
						'activation':'focus',
						'defaultPosition': 'top',
					});
					this_field.css('color','#cc0000');
					this_field.focus();
				}else{
					this_field.css('color','#000');
					$('#tiptip_holder').remove();
				}
			});
		},

		/**
		 * Init option page repeater.
		 */
		optionPageRepeater: function(){
			if( $( '#wpf_et_option_page .wpf-repeater' ).length ){

				$( '#wpf_et_option_page .wpf-repeater' ).repeater(
					{
						repeaters: [{
							selector: '.wpf-multi-key-crm',
		
							show: function () {
								$( this ).slideDown();
								$( '.select4-container',$( this ) ).remove();
								$( '.trigger option:first',$( this ) ).attr( 'selected',false ).attr( 'selected',true );
								$( '.wpf-preview .event-name,.wpf-preview .event-value,.wpf-preview .event-key',$( this ) ).html( '' );
							},
							hide: function (deleteElement) {
								$( this ).slideUp( deleteElement );
							},
						}],
						initEmpty: false,
						show: function () {
		
							$( this ).show();
							var main_parent = $( this ).parents( '.wpf-repeater' );
							$( '.order span',main_parent ).each(
								function (i) {
									var numbering = i + 1;
									$( this ).text( numbering );
								}
							);
		
							// fixing the issue of the labels
							var params = [this];
							$( this ).find( "label[for]" ).each(
								function(index, element) {
									var currentRepeater = params[0];
									var originalFieldId = $( element ).attr( "for" );
									var newField        = $( currentRepeater ).find( "select[id='" + originalFieldId + "']" );
									if ($( newField ).length > 0) {
										var newFieldName = $( newField ).attr( 'name' );
										$( newField ).attr( 'id', newFieldName );
										$( currentRepeater ).find( "label[for='" + originalFieldId + "']" ).attr( 'for', newFieldName );
									}
								},
								params
							);
		
							// Fix select4
							$( '.select4-container',$( this ) ).remove();
							$( '.trigger option:first',$( this ) ).attr( 'selected',false ).attr( 'selected',true );
							$( '.wpf-preview .event-name,.wpf-preview .event-value,.wpf-preview .event-key',$( this ) ).html( '' );
		
							// Nested Repeaters
							$('.nr-item',$( this )).not(':first').remove();
		
						},
						hide: function (deleteElement) {
		
							if($( this ).hasClass('nr-item')){
								$( this ).slideUp( deleteElement );
								wpfEventTracking.updateMultiKeyTable(this,true);
							}else{
								if (confirm( 'Are you sure you want to delete this trigger?' )) {
									$( this ).slideUp( deleteElement );
								}
							}
		
						},
						ready: function (setIndexes) {
							$( ".wpf-repeater .table_body" ).on(
								"sortupdate",
								function( event, ui ) {
									setIndexes();
								}
							);
						},
						isFirstItemUndeletable: false
					}
				);
		
				$( ".wpf-repeater .table_body" ).sortable(
					{
						handle: '.order',
						update: function (event, ui) {
							$( '.order span',$( this ).parents( '.wpf-repeater' ) ).each(
								function (i) {
									var numbering = i + 1;
									$( this ).text( numbering );
								}
							);
						}
					}
				);
		
				// Handle CRMs with character limits in the event names.

		
				if ( wpfEventData.crm === 'activecampaign' ) {
					wpfEventTracking.limitCharachters(40);
				}
		
				if ( wpfEventData.crm === 'mailchimp' ) {
					wpfEventTracking.limitCharachters(30);
				}
			}
		},

		/**
		 * Convert select shortcodes to real values.
		 * @param {object} input 
		 * @returns string
		 */
		convertShortcode:function(input){
			// Build up the text values and replacements.
			var input = $(input);
			var trigger     = input.closest( '.wpf-event-tracking' ).attr( 'data-trigger' );
			var integration = input.closest( '.wpf-event-tracking' ).attr( 'data-integration' );
	
			var previews = [];
	
			wpfEventData.triggers[ integration ][ trigger ].options.forEach( function( element ) {
	
				element.options.forEach( function( option ) {
	
					previews.push( {
						text: '{' + element.type + ':' + option.meta + '}',
						preview: option.preview,
					} );
	
				} );
			} );
	
			
			var text_value = input.val();
	
			// Convert shortcodes
			if( text_value.includes('{') && text_value.includes('}') ) {
	
				var matches = text_value.split('{')
				.filter(function(v){ return v.indexOf('}') > -1})
				.map( function(value) { 
					return value.split('}')[0]
				});
				matches.forEach(function(match){
	
					let new_match = '{' + match + '}';
	
					previews.forEach( function( element ) {
	
						if ( element.text == new_match ) {
							text_value = text_value.replace( new_match, '<b>' + element.preview + '</b>');
						}
	
					} );
	
				});
	
			}
	
			return text_value;
		},

		/**
		 * Update the preview HTML table for multi key crm.
		 * @param {object} input 
		 * @param {boolean} remove 
		 */
		updateMultiKeyTable:function(input,remove=false){
			var main_parent = $(input).closest('.wpf-event-tracking');
			var table = main_parent.find('table');
			var tbody = $("<tbody></tbody>");
			
			$(".nr-item",main_parent).each(function(index) {
				if(remove == true && $(input).index() === index){
					return true;
				}
	
				var key = $(this).find(".wpf-et-key").val();
				var value = wpfEventTracking.convertShortcode($(this).find(".wpf-et-value"));
	
				key = (key ? key +":" : '');

				var row = $("<tr>");
				var keyCell = $("<td>").text(key);
				var valueCell = $("<td>").html(value);
				row.append(keyCell).append(valueCell);
				tbody.append(row);
			});
	
			table.html(tbody);
		},

		/**
		 * Show input preview for event names and values.
		 */
		inputPreview:function(){
			if ( ! $(this).val() ) {
				return;
			}
			var input = $(this);
			var text_value = wpfEventTracking.convertShortcode(input);
			var is_multi_key = input.parents('.wpf-event-tracking').find('.wpf-multi-key-crm').length > 0;
	
			if( text_value != '' ){
				input.closest('.wpf-event-tracking').find('.wpf-preview').css('display', 'flex');
			}
	
			if( input.attr('class') == 'wpf-et-name' ){
				if(is_multi_key){
					input.parents('.wpf-event-tracking').find('.event-name').html( text_value );
				}else{
					input.parents('.wpf-event-tracking-input-container').siblings('.wpf-preview').find('.event-name').html( text_value );
				}
			} else if( input.attr('class') == 'wpf-et-key' ){
	
				// We'll make the key lowercase and replace spaces with underscores.
				text_value = text_value.replace(/ /g, '_');
				text_value = text_value.toLowerCase();
	
				input.val( text_value );
	
				input.closest('.wpf-event-tracking-input-container').parent().find('.wpf-preview .event-key').html( text_value + ':' );
				
			} else if( input.attr('class') == 'wpf-et-value' ){
				input.closest('.wpf-event-tracking-input-container').parent().find('.wpf-preview .event-value').html( text_value );
			}
	
			if(is_multi_key){
				wpfEventTracking.updateMultiKeyTable(input);
			}

		},

		/**
		 * Init select4 selector with shortcodes.
		 * @param {object} e 
		 */
		selectorShortcodes:function(e){
			e.preventDefault();
			var in_nested_repeater = false;
			if($(this).parents('.nr-item').length > 0){
				in_nested_repeater = true;
			}
		
	
			var inputContainer = $(this).closest( 'span.wpf-event-tracking-input-container' );
			var select = inputContainer.next( 'select.select4-event-tracking' );
	
			if ( ! select.data( 'select4' ) ) {
	
				var trigger     = $(this).closest( '.wpf-event-tracking' ).attr( 'data-trigger' );
				var integration = $(this).closest( '.wpf-event-tracking' ).attr( 'data-integration' );
	
				var data = []
	
				wpfEventData.triggers[ integration ][ trigger ].options.forEach( function( element ) {
	
					var group = {
						text: element.name,
						children: [ '' ] // for some reason the first item isn't selectable so we'll just add an empty array key here.
					}
	
					element.options.forEach( function( option ) {
	
						var text = '{' + element.type + ':' + option.meta + '}';
	
						group.children.push(
							{
								id: text,
								text: text,
							}
						);
	
					} );
	
					data.push( group );
	
				} );
	
				select.select4({
					allowClear: true,
					theme: 'wpf-events',
					width: '250px',
					data: data,
			
				}).on('select4:select', function(event) {
	
					let value = event.params.data.text;
					let text_field = inputContainer.find('input[type=text]');
					let text_value = text_field.val();
					let cursorPos = text_field.prop('selectionStart');
	
					if ( cursorPos > 0 || ( cursorPos == 0 && text_value.length == 0 ) ) {
	
						// If it's empty or we're inserting at the cursor.
						var textBefore = text_value.substring(0,  cursorPos);
						var textAfter  = text_value.substring(cursorPos, text_value.length);
	
					} else {
	
						// If we're appending to the end.
						var textBefore = text_value;
						var textAfter  = '';
	
					}
	
					
					text_field.val(textBefore.trim() + ' ' + value + textAfter);
					text_field.focus().trigger('change');
					select.val([]); // clear the select
				});
	
			}
	
			select.select4('open');
	
			if(in_nested_repeater){
				$('.select4-dropdown').css({"left":'-56px'});
			}else{
				$('.select4-dropdown').css({"left":'0'});
			}
		
		}
	};


	wpfEventTracking.init();

} );
