<?php

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly
}

class WPF_Logins_BuddyBoss_App {

	/**
	 * Get things started
	 *
	 * @since 1.4.0
	 */
	public function __construct() {

		add_action( 'added_user_meta', array( $this, 'app_first_login' ), 10, 4 );
		add_action( 'updated_user_meta', array( $this, 'app_login' ), 10, 4 );

	}

	/**
	 * Handles BuddyBoss app first login tagging.
	 *
	 * @since 1.4.0
	 *
	 * @param int    $meta_id Meta ID.
	 * @param int    $user_id User ID.
	 * @param string $meta_key Meta key.
	 * @param mixed  $meta_value Meta value.
	 */
	public function app_first_login( $meta_id, $user_id, $meta_key, $meta_value ) {

		if ( '_bbapp_jwt_jti' !== $meta_key ) {
			return;
		}

		$this->app_login( $meta_id, $user_id, $meta_key, $meta_value );

		if ( ! empty( wpf_get_option( 'bb_app_first_login_apply_tags' ) ) ) {
			wp_fusion()->user->apply_tags( wpf_get_option( 'bb_app_first_login_apply_tags' ), $user_id );
		}

	}

	/**
	 * Handles BuddyBoss app login date syncing.
	 *
	 * @since 1.4.0
	 *
	 * @param int    $meta_id Meta ID.
	 * @param int    $user_id User ID.
	 * @param string $meta_key Meta key.
	 * @param mixed  $meta_value Meta value.
	 */
	public function app_login( $meta_id, $user_id, $meta_key, $meta_value ) {

		if ( '_bbapp_jwt_jti' !== $meta_key ) {
			return;
		}

		if ( is_array( $meta_value ) ) {

			$first_item = reset( $meta_value );

			// Check if the first item is an array and contains the 'time' key
			if ( is_array( $first_item ) && isset( $first_item['time'] ) ) {
				wp_fusion()->user->push_user_meta( $user_id, array( 'wpf_bb_app_last_login' => wpf_get_iso8601_date( $first_item['time'] ) ) );
			}
		}
	}
}
