document.addEventListener('DOMContentLoaded', function() {
	// Apply tags
	function applyTags(tags) {
	  if (typeof tags === 'object') {
		tags.forEach(function(tag, index) {
		  if (tag.slice(-1) === ',') {
			tags[index] = tag.slice(0, -1);
		  }
		});
	  }
  
	  if (typeof tags === 'string') {
		if (tags.slice(-1) === ',') {
		  tags = tags.slice(0, -1);
		}
	  }
  
	  var xhr = new XMLHttpRequest();
	  var url = wpf_mt_ajax.ajaxurl;
	  var params = 'action=apply_tags&tags=' + encodeURIComponent(tags);
  
	  xhr.open('POST', url, true);
	  xhr.setRequestHeader('Content-type', 'application/x-www-form-urlencoded');
  
	  xhr.onreadystatechange = function() {
		if (xhr.readyState === 4 && xhr.status === 200) {
		  // Request completed successfully
		}
	  };
  
	  xhr.send(params);
	}
  
	function sortNumber(a, b) {
	  return a - b;
	}
  
	// Get data attributes from player
	function getPlayerData(el) {

		var playerData = {};
		var dataset = el.dataset;
	  
		// Convert dataset to regular object
		Object.keys(dataset).forEach(function (key) {
		  playerData[key] = dataset[key];
		});
	  
		// Build object of registered timecodes and tags
		if (
		  typeof playerData.wpf_tags_timecode !== 'undefined' &&
		  typeof playerData.wpf_timecode !== 'undefined'
		) {
		  playerData.timecodeTags = {};
	  
		  Object.keys(playerData).forEach(function (index) {
			if (index.startsWith('wpf_timecode')) {
			  // Get index of timecode
			  var i = index.replace('wpf_timecode', '');
	  
			  // Convert mm:ss format to seconds
			  var pieces = playerData[index].toString().split(':');
	  
			  if (Array.isArray(pieces)) {
				var seconds = pieces.pop();
				pieces.forEach(function (pieceVal) {
				  seconds = parseFloat(seconds) + pieceVal * 60;
				});
			  } else {
				var seconds = pieces;
			  }
	  
			  playerData.timecodeTags[seconds] =
				playerData['wpf_tags_timecode' + i];
			}
		  });
		}
	  
		return playerData;
	  }
  
	// Mediaelement.js embeds
	var videos = document.querySelectorAll('video');
	videos.forEach(function(video) {
	  var playerData = getPlayerData(video);
  
	  if (
		typeof playerData.wpf_tags_start === 'undefined' &&
		typeof playerData.wpf_tags_complete === 'undefined' &&
		typeof playerData.timecodeTags === 'undefined'
	  ) {
		return;
	  }
  
	  new MediaElement(video.getAttribute('id'), {
		success: function(mediaElement, domObject) {
		  if (typeof playerData.wpf_tags_start !== 'undefined') {
			mediaElement.addEventListener('play', function(e) {
			  // mediaElement start video
			  applyTags(playerData.wpf_tags_start);
			});
		  }
  
		  if (typeof playerData.wpf_tags_complete !== 'undefined') {
			mediaElement.addEventListener('ended', function(e) {
			  // mediaElement video complete
			  applyTags(playerData.wpf_tags_complete);
			});
		  }
  
		  if (typeof playerData.timecodeTags !== 'undefined') {
			mediaElement.addEventListener('timeupdate', function(e) {
			  // mediaElement player progress
			  if (Object.keys(playerData.timecodeTags).length === 0) return;
  
			  Object.entries(playerData.timecodeTags).forEach(function(
				timecodeEntry
			  ) {
				var timecode = timecodeEntry[0];
				var tags = timecodeEntry[1];
  
				if (mediaElement.currentTime >= timecode) {
				  applyTags(tags);
				  delete playerData.timecodeTags[timecode];
				}
			  });
			});
		  }
		}
	  });
	});
  

	function initializeBunnyPlayer(el) {

		var playerData = getPlayerData(el);
		var player = new playerjs.Player(el);

		// Set it as ready
		el.dataset.wpfInit = true;
	
		if (typeof playerData.wpf_tags_start !== 'undefined') {
			var startDone = false;
	
			player.on('ready', () => {
				player.on('play', () => {
					// Bunny started video
					if (startDone === false) {
						startDone = true;
						applyTags(playerData.wpf_tags_start);
					}
				});
			});

		}
	
		if (typeof playerData.wpf_tags_complete !== 'undefined') {
			var completeDone = false;
	
			player.on('ready', () => {
				player.on('ended', () => {
					// Bunny completed video
					if (completeDone === false) {
						completeDone = true;
						applyTags(playerData.wpf_tags_complete);
					}
				});
			});

		}

		if (typeof playerData.timecodeTags !== 'undefined') {

			player.on('ready', () => {
				player.on('timeupdate', (data) => {
					if (Object.keys(playerData.timecodeTags).length === 0) {
						return;
					}

					Object.entries(playerData.timecodeTags).forEach(function(
					timecodeEntry
					) {
						var timecode = timecodeEntry[0];
						var tags = timecodeEntry[1];

						if (timecode.endsWith('%')) {
							// Percentage based tracking
							var iframe_precentage = parseInt(timecode.replace('%', ''));
							let video_percentage = (data.seconds / data.duration) * 100;

							if (video_percentage >= iframe_precentage) {
								applyTags(tags);
								delete playerData.timecodeTags[timecode];
							}
						} else {
							if (data.seconds >= timecode) {
								applyTags(tags);
								delete playerData.timecodeTags[timecode];
							}
						}

					});
				});

			});


		}
  
	}


	// Initialize Vimeo player tracking on an element
	function initializeVimeo(el) {
	  var playerData = getPlayerData(el);
	  var player = new Vimeo.Player(el);
  
	  // Set it as ready
	  el.dataset.wpfInit = true;
  
	  if (typeof playerData.wpf_tags_start !== 'undefined') {
		var startDone = false;
  
		player.on('play', function() {
		  // Vimeo started video
		  if (startDone === false) {
			startDone = true;
			applyTags(playerData.wpf_tags_start);
		  }
		});
	  }
  
	  if (typeof playerData.wpf_tags_complete !== 'undefined') {
		var completeDone = false;
  
		player.on('timeupdate', function(data) {
		  if (data.percent >= 0.95 && completeDone === false) {
			// Vimeo 95% complete
			completeDone = true;
			applyTags(playerData.wpf_tags_complete);
		  }
		});
	  }
  
	  if (typeof playerData.timecodeTags !== 'undefined') {
		player.on('timeupdate', function(data) {
		  // Vimeo player progress
		  if (Object.keys(playerData.timecodeTags).length === 0) {
			return;
		  }
  
		  Object.entries(playerData.timecodeTags).forEach(function(
			timecodeEntry
		  ) {
			var timecode = timecodeEntry[0];
			var tags = timecodeEntry[1];
  
			if (timecode.endsWith('%')) {
			  // Percentage based tracking
			  var percent = parseInt(timecode.replace('%', ''));
			  percent = percent / 100;
  
			  if (data.percent >= percent) {
				applyTags(tags);
				delete playerData.timecodeTags[timecode];
			  }
			} else {
			  if (data.seconds >= timecode) {
				applyTags(tags);
				delete playerData.timecodeTags[timecode];
			  }
			}
		  });
		});
	  }
	}
  
	// Initialize YouTube player tracking on an element
	function initializeYouTube(el) {
	  if (typeof YT === 'undefined') {
		console.log('YouTube API Undefined. Unable to initialize WP Fusion media tracking.');
		return;
	  }
  
	  var playerData = getPlayerData(el);
	  var playerId = el.getAttribute('id');
  
	  if (!playerId) {
		playerId = 'ytplayer' + Math.floor(Math.random() * 100);
		el.setAttribute('id', playerId);
	  }
  
	  var appliedStart = false;
	  var player = YT.get(playerId);
  
	  if (typeof player === 'undefined') {
		// Create a new player
		player = new YT.Player(playerId, {
		  events: {
			onReady: onPlayerReady,
			onStateChange: onPlayerStateChange
		  },
		  videoId: el.getAttribute('src').split('/embed/')[1].split('?')[0]
		});
	  } else {
		// The player was created by another plugin
		player.addEventListener('onReady', onPlayerReady);
		player.addEventListener('onStateChange', onPlayerStateChange);
	  }
  
	  // Set it as ready
	  el.dataset.wpfInit = true;
  
	  // Elementor image overlay support
	  var figure = el.closest('figure');
  
	  if (figure) {
		figure.addEventListener('click', function(event) {
		  if (typeof playerData.wpf_tags_start !== 'undefined') {
			applyTags(playerData.wpf_tags_start);
			appliedStart = true;
		  }
		});
	  }
  
	  function onPlayerStateChange(event) {
		if (
		  event.data === YT.PlayerState.PLAYING &&
		  typeof playerData.wpf_tags_start !== 'undefined'
		) {
		  // YouTube started video
		  if (appliedStart === false) {
			var tags = playerData.wpf_tags_start;
  
			if (tags.includes(',')) {
			  tags = tags.split(',');
			}
  
			applyTags(tags);
			appliedStart = true;
		  }
		} else if (
		  event.data === YT.PlayerState.ENDED &&
		  typeof playerData.wpf_tags_complete !== 'undefined'
		) {
		  // YouTube finished video
		  // This doesn't currently work when both FitVids (Boss Theme) and Elementor are in use at the same time
		  applyTags(playerData.wpf_tags_complete);
		}
	  }
  
	  function onPlayerReady(event) {
		if (typeof playerData.timecodeTags !== 'undefined') {
		  var videotime = 0;
		  var timeupdater = null;
  
		  function updateTime() {
			var oldTime = videotime;
  
			if (player && player.getCurrentTime) {
			  videotime = player.getCurrentTime();
			}
  
			if (videotime !== oldTime) {
			  onProgress(videotime);
			}
		  }
  
		  timeupdater = setInterval(updateTime, 500);
		}
	  }
  
	  function onProgress(currentTime) {
		var progressTags = [];
  
		if (
		  typeof playerData.wpf_tags_start !== 'undefined' &&
		  appliedStart === false
		) {
		  var tags = playerData.wpf_tags_start.split(',');
  
		  progressTags = progressTags.concat(tags);
		  appliedStart = true;
		}
  
		// YouTube 500ms player progress
		if (Object.keys(playerData.timecodeTags).length > 0) {
		  Object.entries(playerData.timecodeTags).forEach(function(timecodeEntry) {

			var timecode = timecodeEntry[0];
			var tags = timecodeEntry[1];
  
			if (timecode.endsWith('%')) {
			  // Percentage based tracking
			  var playingPercentage = currentTime / player.getDuration();
			  var percent = parseInt(timecode.replace('%', ''));
			  percent = percent / 100;
  
			  if (playingPercentage >= percent) {
				progressTags = progressTags.concat(tags);
				delete playerData.timecodeTags[timecode];
			  }
			} else {
			  if (currentTime >= timecode) {
				progressTags = progressTags.concat(tags);
				delete playerData.timecodeTags[timecode];
			  }
			}
		  });
		}
  
		if (progressTags.length) {
		  applyTags(progressTags);
		}
	  }
	}
  
	// Initialize Wistia player tracking on an element
	function initializeWistia(el) {
	  var playerData = getPlayerData(el);
	  var videoObject = Wistia.api(el.getAttribute('id'));
  
	  // Set it as ready
	  el.dataset.wpfInit = true;
  
	  videoObject.bind('play', function() {
		if (typeof playerData.wpf_tags_start !== 'undefined') {
		  var startDone = false;
  
		  // Wistia start video
		  if (startDone === false) {
			startDone = true;
			applyTags(playerData.wpf_tags_start);
		  }
		}
	  });
  
	  videoObject.bind('timechange', function(t) {
		if (typeof playerData.wpf_tags_complete !== 'undefined') {
		  var completeDone = false;
  
		  if (t / videoObject.duration() >= 0.95 && completeDone === false) {
			// Wistia 95% complete
			completeDone = true;
			applyTags(playerData.wpf_tags_complete);
		  }
		}
  
		if (typeof playerData.timecodeTags !== 'undefined') {
		  Object.entries(playerData.timecodeTags).forEach(function(timecodeEntry) {
			var timecode = timecodeEntry[0];
			var tags = timecodeEntry[1];
  
			// Wistia player progress
			if (t >= timecode) {
			  applyTags(tags);
			  delete playerData.timecodeTags[timecode];
			}
		  });
		}
	  });
	}


	jQuery(document).one('fv_player_loaded',function(){
		let players = document.querySelectorAll('.flowplayer');
		players.forEach(function(player) {
			var api = flowplayer( player);
			initializeFvPlayer(player,api);
		});
	});

	// Initialize FV player tracking on shortcodes.
	function initializeFvPlayer(player,api){
		var playerData = getPlayerData(player);
		var appliedStart = false;

		//Start tags / Timecodes.
		api.on( 'progress', function( e, api, currentTime ) {
			var progressTags = [];
		
			if (
				typeof playerData.wpf_tags_start !== 'undefined' &&
				appliedStart === false
			  ) {
				var tags = playerData.wpf_tags_start.split(',');
		
				progressTags = progressTags.concat(tags);
				appliedStart = true;
			  }


			if (Object.keys(playerData.timecodeTags).length > 0) {
				Object.entries(playerData.timecodeTags).forEach(function(timecodeEntry) {
	
				var timecode = timecodeEntry[0];
				var tags = timecodeEntry[1];
		
				if (timecode.endsWith('%')) {
					// Percentage based tracking
					var playingPercentage = currentTime / player.getDuration();
					var percent = parseInt(timecode.replace('%', ''));
					percent = percent / 100;
		
					if (playingPercentage >= percent) {
					progressTags = progressTags.concat(tags);
					delete playerData.timecodeTags[timecode];
					}
				} else {
					if (currentTime >= timecode) {
					progressTags = progressTags.concat(tags);
					delete playerData.timecodeTags[timecode];
					}
				}
				});
			}
		
			if (progressTags.length) {
				applyTags(progressTags);
			}
			  
		});

		
		
		// Complete tags.
		api.on( 'finish', function( player ) {
			if(typeof playerData.wpf_tags_complete !== 'undefined'){
				applyTags(playerData.wpf_tags_complete);
			}
		});

	}
  

	// Initialize VooPlayer player tracking on an element
	function initializeVooPlayer(el) {
	  var playerData = getPlayerData(el);
	  var playerId = el.getAttribute('data-playerid');
  
	  document.addEventListener('vooPlayerReady', function(event) {
		// Set it as ready
		el.dataset.wpfInit = true;
  
		var completeTimecode = false;
		var startDone = false;
		var completeDone = false;
		var vooTimecodes = [1];
  
		// Merge in timecodes
		if (
		  typeof playerData.timecodeTags !== 'undefined' &&
		  Object.keys(playerData.timecodeTags).length !== 0
		) {
		  Object.entries(playerData.timecodeTags).forEach(function(timecodeEntry) {
			var timecode = timecodeEntry[0];
			var tags = timecodeEntry[1];
  
			vooTimecodes.push(parseInt(timecode));
		  });
		}
  
		vooAPI(playerId, 'getDuration', ['1'], function(duration) {
		  completeTimecode = parseInt(Math.floor(duration * 0.95));
		  vooTimecodes.push(completeTimecode);
  
		  vooTimecodes.sort(sortNumber);
		});
  
		function checkVooProgress() {
		  vooAPI(playerId, 'getTime', vooTimecodes, checkTimecodes);
  
		  function checkTimecodes(time) {
			if (typeof playerData.wpf_tags_start !== 'undefined') {
			  if (startDone === false) {
				startDone = true;
				applyTags(playerData.wpf_tags_start);
  
				vooTimecodes.splice(vooTimecodes.indexOf('1'), 1);
  
				vooAPI(playerId, 'getTime', vooTimecodes, checkTimecodes);
			  }
			}
  
			if (typeof playerData.wpf_tags_complete !== 'undefined') {
			  if (completeDone === false && time === completeTimecode) {
				completeDone = true;
				applyTags(playerData.wpf_tags_complete);
				clearInterval(vooProgressChecker);
			  }
			}
  
			if (typeof playerData.timecodeTags !== 'undefined') {
			  if (Object.keys(playerData.timecodeTags).length === 0) {
				return;
			  }
  
			  Object.entries(playerData.timecodeTags).forEach(function(
				timecodeEntry
			  ) {
				var timecode = timecodeEntry[0];
				var tags = timecodeEntry[1];
  
				if (timecode == time) {
				  applyTags(tags);
				  delete playerData.timecodeTags[timecode];
  
				  vooTimecodes.splice(vooTimecodes.indexOf(timecode), 1);
  
				  vooAPI(playerId, 'getTime', vooTimecodes, checkTimecodes);
				}
			  });
			}
		  }
		}
  
		var vooProgressChecker = setInterval(checkVooProgress, 1000);
	  });
	}
  
	// Generic initializer. Checks the type and calls the right function
	function initializePlayer(el) {
	  var playerType = el.getAttribute('data-player_type');

	  if (typeof playerType === 'undefined') {
		return; // If WP Fusion isn't set on this player
	  }
  
	  if (playerType === 'vimeo') {
		initializeVimeo(el);
	  } else if (playerType === 'youtube') {
		initializeYouTube(el);
	  } else if (playerType === 'wistia') {
		initializeWistia(el);
	  } else if (playerType === 'vooplayer') {
		initializeVooPlayer(el);
	  } else if (playerType === 'bunny') {
		initializeBunnyPlayer(el);
	  }
	}
  
	// Try to find videos to interact with
  
	// To start, on page load we'll go through all the iframes and see if any are eligible
	function initializeIFrames() {
	  var iframes = document.querySelectorAll('iframe');
	  iframes.forEach(function(iframe) {
		initializePlayer(iframe);
	  });

	  var wistia_embeds = document.querySelectorAll('.wistia_embed');
	  if(wistia_embeds.length){
		wistia_embeds.forEach(function(embed) {
			initializeWistia(embed);
		  });
	  }

	}
  
	setTimeout(initializeIFrames, 1000);
  
	// PrestoPlayer
	if (typeof wp !== 'undefined' && typeof wp.hooks !== 'undefined') {
	  wp.hooks.addAction(
		'presto.playerReady',
		'wp-fusion-media-tools',
		function(player) {

		  var container = ( player.elements.container.getRootNode().host || player.elements.container ).closest('figure');
		  var playerData = getPlayerData(container);
  
		  if (typeof playerData.wpf_tags_start !== 'undefined') {
			// Started playing
			var startDone = false;
  
			player.on('playing', function(event) {
			  if (startDone === false) {
				var player = event.detail.plyr;
  
				startDone = true;
				applyTags(playerData.wpf_tags_start);
			  }
			});
		  }
  
		  // Completed
		  var completeDone = false;
  
		  player.on('timeupdate', function(event) {
			var player = event.detail.plyr;
  
			if (typeof playerData.wpf_tags_complete !== 'undefined') {
			  if (player.currentTime / player.duration >= 0.95 && completeDone === false) {
				// 95% complete
				completeDone = true;
				applyTags(playerData.wpf_tags_complete);
			  }
			}
  
			if (typeof playerData.timecodeTags !== 'undefined') {
			  Object.entries(playerData.timecodeTags).forEach(function(timecodeEntry) {
				var timecode = timecodeEntry[0];
				var tags = timecodeEntry[1];
  
				if (timecode.endsWith('%')) {
				  // Percentage based tracking
				  var percent = parseInt(timecode.replace('%', ''));
				  percent = percent / 100;
  
				  if (player.currentTime / player.duration >= percent) {
					applyTags(tags);
					delete playerData.timecodeTags[timecode];
				  }
				} else {
				  if (player.currentTime >= timecode) {
					applyTags(tags);
					delete playerData.timecodeTags[timecode];
				  }
				}
			  });
			}
		  });
		}
	  );
	}
	

	// Elementor lightbox for youtube, add the enablejsapi so we can use API to track it.
	if(jQuery('.elementor-custom-embed-image-overlay').length > 0){
		jQuery.each(jQuery('.elementor-custom-embed-image-overlay'),function(i,element){
			let lightbox = JSON.parse(jQuery(element).attr('data-elementor-lightbox'));
			if(lightbox.videoType === 'youtube'){
				lightbox.url = lightbox.url + '&enablejsapi=1';
				jQuery(element).attr('data-elementor-lightbox',JSON.stringify(lightbox));
			}
		});
	}


	// Elementor lightbox, add wpf attrs to track it.
	jQuery(document).on('click','.elementor-custom-embed-image-overlay',function(){
		let attrs = this.attributes;
		let intervalID = setInterval(function() {
			let lightbox = jQuery('.elementor-lightbox iframe');
			if(lightbox.length > 0){
				for(let attr of attrs) {
					if(attr.name === 'data-player_type' || attr.name.startsWith('data-wpf')){
						lightbox.attr(attr.name, attr.value);
					}
				}
				initializeIFrames();
				clearInterval(intervalID);
			}
		}, 10);
	});




  });