<?php

class WPF_Media_Tools_Public {

	/**
	 * Get things started
	 *
	 * @since 1.0
	 * @return void
	 */

	public function __construct() {

		// Video shortcode filtering
		add_filter( 'shortcode_atts_video', array( $this, 'video_atts' ), 10, 4 );
		add_filter( 'wp_video_shortcode', array( $this, 'shortcode_output_video' ), 10, 5 );

		// iFrame player filtering (Bunny.net)
		add_filter( 'the_content', array( $this, 'iframe_player_filter' ) );

		// Embed shortcode filtering
		add_filter( 'shortcode_atts_embed', array( $this, 'video_atts' ), 10, 4 );
		add_filter( 'embed_oembed_html', array( $this, 'shortcode_output_embed' ), 10, 4 );

		// Presto Player
		add_filter( 'do_shortcode_tag', array( $this, 'shortcode_output_presto_player' ), 10, 4 );
		add_filter( 'render_block', array( $this, 'block_output_presto_player' ), 10, 2 );

		// VooPlayer
		add_filter( 'do_shortcode_tag', array( $this, 'shortcode_output_vooplayer' ), 10, 4 );

		// FVPlayer
		add_filter( 'do_shortcode_tag', array( $this, 'shortcode_output_fvplayer' ), 10, 4 );

		// s3 Media Maestro
		add_filter( 'shortcode_atts_s3mm', array( $this, 'video_atts' ), 10, 4 );
		add_filter( 's3mm_shortcode_html', array( $this, 'shortcode_output_video' ), 10, 3 );

		// Frontend scripts
		add_action( 'wp_enqueue_scripts', array( $this, 'enqueue_scripts' ) );

	}

	/**
	 * Add custom shortcode atts to media / embed shortcodes
	 *
	 * @since 1.0
	 * @return array Atts
	 */

	public function video_atts( $out, $pairs, $atts, $shortcode ) {

		foreach ( $atts as $key => $value ) {

			if ( strpos( $key, 'wpf_' ) !== false ) {
				$out[ $key ] = $value;
			}
		}

		return $out;

	}


	/**
	 * Adds data attributes to media shortcode output
	 *
	 * @since 1.0
	 * @return string Output
	 */

	public function shortcode_output_video( $output, $atts, $media, $post_id = false, $library = false ) {

		$data_atts = '';
		$found     = false;

		foreach ( $atts as $key => $value ) {
			if ( strpos( $key, 'wpf_' ) !== false ) {
				$data_atts .= 'data-' . $key . '="' . $value . '" ';
				$found      = true;
			}
		}

		if ( $found == true ) {
			$output = str_replace( '<video ', '<video ' . $data_atts, $output );
			wp_enqueue_script( 'wpf-media-tools' );
		}

		return $output;

	}

	/**
	 * Filters the_content to add data attributes to Bunny.net iFrames.
	 *
	 * @since 1.5.0
	 *
	 * @param string $content The content.
	 * @return string The content.
	 */

	public function iframe_player_filter( $content ) {

		if ( false !== strpos( $content, 'src="https://iframe.mediadelivery.net/embed/' ) && false !== strpos( $content, 'data-wpf_' ) ) {
			$content = str_replace( 'src="https://iframe.mediadelivery.net/embed/', 'data-player_type="bunny" src="https://iframe.mediadelivery.net/embed/', $content );
			wp_enqueue_script( 'bunny-api', '//assets.mediadelivery.net/playerjs/player-0.1.0.min.js', array(), '1.0', true );
			wp_enqueue_script( 'wpf-media-tools' );
		}

		return $content;

	}

	/**
	 * Adds data attributes to embed shortcode output
	 *
	 * @since 1.0
	 * @return string Output
	 */

	public function shortcode_output_embed( $output, $url, $atts, $post_ID ) {

		if ( ! is_array( $atts ) ) {
			return $output;
		}

		$data_atts = '';
		$found     = false;

		foreach ( $atts as $key => $value ) {
			if ( strpos( $key, 'wpf_' ) !== false ) {
				$data_atts .= 'data-' . $key . '="' . $value . '" ';
				$found      = true;
			}
		}

		if ( $found != true ) {
			return $output;
		}

		wp_enqueue_script( 'wpf-media-tools' );

		// Vimeo iFrames
		if ( strpos( $output, 'vimeo' ) !== false ) {

			$data_atts .= 'data-player_type="vimeo" ';
			wp_enqueue_script( 'vimeo-api', '//player.vimeo.com/api/player.js', array(), WPF_MEDIA_TOOLS_VERSION, true );

		} elseif ( strpos( $output, 'youtube' ) !== false ) {

			$output     = str_replace( '?feature=oembed', '?feature=oembed&enablejsapi=1', $output );
			$data_atts .= 'data-player_type="youtube" id="' . uniqid( 'wpf_youtube_' ) . '" ';
			wp_enqueue_script( 'youtube-api', '//www.youtube.com/iframe_api', array(), '1.0', true );

		} elseif ( strpos( $output, 'wistia' ) !== false ) {

			$data_atts .= 'data-player_type="wistia" id="' . uniqid( 'wpf_wistia_' ) . '" ';

			$output = str_replace( 'sandbox="allow-scripts" ', '', $output );
			$output = str_replace( 'security="restricted" ', '', $output );
			$output = str_replace( 'wp-embedded-content', 'wistia_embed', $output );

			wp_enqueue_script( 'wistia-api', '//fast.wistia.com/assets/external/E-v1.js', array(), '1.0', true );

		}

		$output = str_replace( '<iframe ', '<iframe ' . $data_atts, $output );

		return $output;

	}

	/**
	 * Adds data attributes to Presto Player output
	 *
	 * @since 1.2.0
	 *
	 * @param string       $output The shortcode output.
	 * @param string       $tag    The name of the shortcode tag.
	 * @param array|string $attr   Shortcode attributes array or empty string.
	 * @param array        $m      Regular expression match array.
	 * @return string The shortcode output.
	 */
	public function shortcode_output_presto_player( $output, $tag, $attr, $m ) {

		if ( 'presto_player' !== $tag ) {
			return $output;
		}

		$data_atts = '';
		$proceed   = false;

		// Check if WPF is in use on the video
		foreach ( $attr as $key => $value ) {

			if ( strpos( $key, 'wpf_' ) !== false ) {
				$data_atts .= 'data-' . $key . '="' . $value . '" ';
				$proceed    = true;
			}
		}

		if ( ! $proceed ) {
			return $output;
		}

		wp_enqueue_script( 'wpf-media-tools' );

		$data_atts .= 'data-player_type="presto_player" id="' . uniqid( 'wpf_presto_player_' ) . '" ';

		$output = str_replace( '<figure ', '<figure ' . $data_atts, $output );

		return $output;
	}

	/**
	 * Adds data attributes to Presto Player block content
	 *
	 * @since 1.2.6
	 *
	 * @param string $block_content    The block content.
	 * @param array  $block            The block properties.
	 * @return array|string|string[]
	 */
	public function block_output_presto_player( $block_content, $block ) {

		$presto_blocks = array(
			'presto-player/self-hosted',
			'presto-player/youtube',
			'presto-player/bunny',
			'presto-player/vimeo',
		);

		$core_blocks = array(
			'core/video',
			'core/embed',
		);

		$blocks = array_merge( $presto_blocks, $core_blocks );

		if ( ! in_array( $block['blockName'], $blocks, true ) ) {
			return $block_content;
		}

		$data_atts = '';
		$proceed   = false;
		$attr      = $block['attrs'];

		// Check if WPF is in use on the video.
		foreach ( $attr as $key => $value ) {

			if ( strpos( $key, 'wpf_' ) !== false ) {
				if ( $key === 'wpf_timecode_tags' && is_array( $value ) ) {
					foreach ( $value as $item ) {
						if ( $item['id'] == 0 ) {
							$data_atts .= 'data-' . 'wpf_timecode' . '="' . $item['timecode'] . '" ';

							if ( isset( $item['tags'] ) && is_array( $item['tags'] ) ) {
								$tags = '';

								foreach ( $item['tags'] as $tag ) {
									$tags .= $tag['label'] . ',';
								}

								$data_atts .= 'data-' . 'wpf_tags_timecode' . '="' . $tags . '" ';
							} else {
								if ( isset( $item['tags'] ) ) {
									$data_atts .= 'data-' . 'wpf_tags_timecode' . '="' . $item['tags'] . '" ';
								}
							}
						} else {
							if ( isset( $item['tags'] ) ) {
								$data_atts .= 'data-' . 'wpf_timecode_' . $item['id'] . '="' . $item['timecode'] . '" ';

								if ( is_array( $item['tags'] ) ) {
									$tags = '';

									foreach ( $item['tags'] as $tag ) {
										$tags .= $tag['label'] . ',';
									}

									$data_atts .= 'data-' . 'wpf_tags_timecode_' . $item['id'] . '="' . $tags . '" ';
								} else {
									$data_atts .= 'data-' . 'wpf_tags_timecode_' . $item['id'] . '="' . $item['tags'] . '" ';
								}
							}
						}
					}
				} elseif ( is_array( $value ) ) {
					$tags = '';

					foreach ( $value as $tag ) {
						$tags .= $tag['label'] . ',';
					}

					$data_atts .= 'data-' . $key . '="' . $tags . '" ';
				} else {
					$data_atts .= 'data-' . $key . '="' . $value . '" ';
				}

				$proceed = true;
			}
		}

		if ( ! $proceed ) {
			return $block_content;
		}

		wp_enqueue_script( 'wpf-media-tools' );

		$player_type = $this->get_player_type( $block, $presto_blocks );

		$html_element = '<figure';

		if ( in_array( $block['blockName'], $presto_blocks ) ) {

			$data_atts .= 'data-player_type="' . $player_type . '" id="' . uniqid( 'wpf_presto_player_' ) . '"';

		} elseif ( in_array( $block['blockName'], $core_blocks ) ) {

			$data_atts .= 'data-player_type="' . $player_type . '" id="' . uniqid( 'wpf_core_block_player_' ) . '"';

			if ( $block['blockName'] === 'core/video' ) {
				$html_element = '<video';
			}

			if ( $block['blockName'] === 'core/embed' ) {
				$html_element = '<iframe';

				switch ( $this->get_player_type( $block, $presto_blocks ) ) {
					case 'vimeo':
						wp_enqueue_script( 'vimeo-api', '//player.vimeo.com/api/player.js' );
						break;
					case 'youtube':
						wp_enqueue_script( 'youtube-api', '//www.youtube.com/iframe_api', array(), '1.0', false );
						$block_content = str_replace( 'feature=oembed', 'feature=oembed&enablejsapi=1', $block_content );
						break;
				}
			}
		}

		return str_replace( $html_element . ' ', $html_element . ' ' . $data_atts, $block_content );
	}



	/**
	 * Adds data attributes to FVPlayer output.
	 *
	 * @since 1.5.0
	 *
	 * @param string       $output The shortcode output.
	 * @param string       $tag    The name of the shortcode tag.
	 * @param array|string $attr   Shortcode attributes array or empty string.
	 * @param array        $m      Regular expression match array.
	 * @return string The shortcode output.
	 */

	public function shortcode_output_fvplayer( $output, $tag, $attr, $m ) {

		if ( 'fvplayer' !== $tag ) {
			return $output;
		}

		$data_atts = '';
		$proceed   = false;

		// Check if WPF is in use on the video
		foreach ( $attr as $key => $value ) {

			if ( strpos( $key, 'wpf_' ) !== false ) {
				$data_atts .= 'data-' . $key . '="' . $value . '" ';
				$proceed    = true;
			}
		}

		if ( ! $proceed ) {
			return $output;
		}

		if ( strpos( $output, 'vimeo.com' ) !== false ) {
			$data_atts .= 'data-player_type="vimeo" id="' . uniqid( 'wpf_fvplayer_' ) . '" ';

			wp_enqueue_script( 'wpf-vimeo-player', WPF_MEDIA_TOOLS_DIR_URL . 'assets/players/vimeo.player.min.js', array(), WPF_MEDIA_TOOLS_VERSION, true );

			$output = str_replace( '<iframe ', '<iframe ' . $data_atts, $output );

		} elseif ( strpos( $output, 'wistia.com' ) !== false ) {
			$data_atts .= 'data-player_type="wistia" id="' . uniqid( 'wpf_fvplayer_' ) . '" ';
			$output     = str_replace( "class='wistia_embed", $data_atts . " class='wistia_embed", $output );

		} else {
			// This for youtube and plain videos uploaded.
			$data_atts .= 'data-player_type="fvplayer" id="' . uniqid( 'wpf_fvplayer_' ) . '" ';
			$output     = str_replace( 'class="flowplayer', $data_atts . ' class="flowplayer', $output );
		}

		wp_enqueue_script( 'wpf-media-tools' );

		return $output;

	}


	/**
	 * Adds data attributes to VooPlayer output
	 *
	 * @since 1.0
	 * @return string Output
	 */

	public function shortcode_output_vooplayer( $output, $tag, $attr, $m ) {

		if ( $tag != 'vooplayer' ) {
			return $output;
		}

		$data_atts = '';
		$proceed   = false;

		// Check if WPF is in use on the video
		foreach ( $attr as $key => $value ) {

			if ( strpos( $key, 'wpf_' ) !== false ) {
				$data_atts .= 'data-' . $key . '="' . $value . '" ';
				$proceed    = true;
			}
		}

		if ( ! $proceed ) {
			return $output;
		}

		wp_enqueue_script( 'wpf-media-tools' );

		$data_atts .= 'data-player_type="vooplayer" id="' . uniqid( 'wpf_vooplayer_' ) . '" ';

		$output = str_replace( '<iframe ', '<iframe ' . $data_atts, $output );

		return $output;

	}

	/**
	 * Add custom shortcode atts to media shortcode
	 *
	 * @since 1.0.0
	 */

	public function enqueue_scripts() {

		wp_register_script( 'wpf-media-tools', WPF_MEDIA_TOOLS_DIR_URL . 'assets/wpf-media-tools.js', array( 'wp-mediaelement' ), WPF_MEDIA_TOOLS_VERSION, true );

		$localize_data = array(
			'ajaxurl' => admin_url( 'admin-ajax.php' ),
		);

		wp_localize_script( 'wpf-media-tools', 'wpf_mt_ajax', $localize_data );

		if ( wp_fusion()->settings->get( 'media_tools_enqueue_scripts' ) ) {
			wp_enqueue_script( 'wpf-media-tools' );

			$players = wp_fusion()->settings->get( 'media_tools_players', array() );

			if ( isset( $players['vimeo'] ) ) {
				wp_enqueue_script( 'vimeo-api', '//player.vimeo.com/api/player.js' );
			}

			if ( isset( $players['youtube'] ) ) {
				wp_enqueue_script( 'youtube-api', '//www.youtube.com/iframe_api', array( 'wpf-media-tools' ), '1.0', false );
			}

			if ( isset( $players['wistia'] ) ) {
				wp_enqueue_script( 'wistia-api', '//fast.wistia.com/assets/external/E-v1.js', array(), '1.0', false );
			}

			if ( isset( $players['bunny'] ) ) {
				wp_enqueue_script( 'bunny-api', '//assets.mediadelivery.net/playerjs/player-0.1.0.min.js', array(), '1.0', true );
			}
		}

	}

	/**
	 * Get the player type based on the block type or embed type
	 *
	 * @since 1.2.7
	 *
	 * @param $block
	 * @param $presto_blocks
	 *
	 * @return mixed|string
	 */
	private function get_player_type( $block, $presto_blocks ) {
		switch ( $block['blockName'] ) {
			case 'core/embed':
				return $block['attrs']['providerNameSlug'];
			case 'core/video':
				return 'self-hosted';
		}

		if ( in_array( $block['blockName'], $presto_blocks ) ) {
			return 'presto_player';
		}

		return '';
	}

}

new WPF_Media_Tools_Public();
