import TimecodeTag from "./timecode-tag";
const { __ } = wp.i18n;
const { BaseControl } = wp.components;

function TagRepeater( props ) {
    const { wpf_timecode_tags } = props.attributes;
    let tags;

    if( ! wpf_timecode_tags || wpf_timecode_tags.length <= 0 ){
        props.setAttributes( {
            wpf_timecode_tags: [
                {
                    id: 0,
                    tag: '',
                    timecode: ''
                }
            ]
        } );
    }

    if ( props.attributes.wpf_timecode_tags.length ) {
        tags = props.attributes.wpf_timecode_tags.map((el, index) => {
            return <TimecodeTag index={ index } {...props} />;
        });
    }

    const addTag = () => {
        const tags = [ ...props.attributes.wpf_timecode_tags ];
        tags.push( {
            id: tags[ tags.length - 1 ].id + 1,
            tag: '',
            timecode: ''
        } );

        props.setAttributes( { wpf_timecode_tags: tags } );
    };

    const repeaterStyle = {
        display: 'flex',
        justifyContent: 'center',
        marginTop: '10px',
        cursor: 'pointer'
    };

    return (
        <>
            <BaseControl
                label={ __( 'Timecodes', 'wp-fusion-media-tools' ) }
                help={ __( 'Enter a timecode (i.e. 0:15) or percent (i.e. 15%), and one or more tags', 'wp-fusion-media-tools' ) }
            />
                { tags }

            <div className="wpf-media-add-new-wrap" style={ repeaterStyle } onClick={addTag}>
                <span className="dashicons dashicons-plus-alt wpf-media-add-new-icon"></span>
                <BaseControl label='Add Item' className='wpf-media-add-new-text' />
            </div>
        </>
    );
}

export default TagRepeater;