import WpfSelect from "@verygoodplugins/wpfselect";

const { __ } = wp.i18n;
const { TextControl } = wp.components;
import { useState, useEffect } from '@wordpress/element';
import apiFetch from '@wordpress/api-fetch';

function TimecodeTag( props ) {
    const [ availableTags, setAvailableTags ] = useState( null );

    useEffect( () => {
        apiFetch( { path: '/wp-fusion/secure-blocks/v1/available-tags/' } ).then(
            ( result ) => {
                setAvailableTags( result );
            } );
    }, [] );

    const id = props.index;

    const setValue = ( name, value ) => {
        const tags = [ ...props.attributes.wpf_timecode_tags ];
        tags[ id ][ name ] = value;
        props.setAttributes( { wpf_timecode_tags: tags } );
    }

    const removeTag = () => {
        const tags = [ ...props.attributes.wpf_timecode_tags ];
        tags.splice(id, 1)
        props.setAttributes( { wpf_timecode_tags: tags } );
    };

    const closeStyles = {
        float : 'right',
        marginTop : '-20px',
        marginRight: '-20px',
        backgroundColor: '#fff',
        cursor: 'pointer'
    }

    return (
        <div style={{
            border: '1px solid #000',
            padding: '10px',
            borderRadius: '5px',
            marginTop: '10px'
        }} className='wpf-media-repeater-item'>
            <span style={closeStyles} className="dashicons dashicons-dismiss wpf-media-repeater-close" onClick={removeTag}></span>

            <div>
                <TextControl
                    value={ ( props.attributes.wpf_timecode_tags[id] !== undefined ) ? props.attributes.wpf_timecode_tags[id].timecode : '' }
                    onChange={(timecodeValue) => {
                        setValue('timecode', timecodeValue);
                    }}
                    placeholder={'0:00'}
                    style={{marginBottom: '-11px !important'}}
                    className='wpf-media-repeater-timecode'
                />
            </div>

            <WpfSelect
                existingTags = { ( props.attributes.wpf_timecode_tags[id] !== undefined ) ? props.attributes.wpf_timecode_tags[id].tags : '' }
                onChange = {
                    ( tagsValue) => {
                        setValue( 'tags', tagsValue )
                    }
                }
                elementID = { 'wpf-timecode-tags-select-' + id }
                { ...props }
                refreshButtonClass = 'wpf-select-refresh-timecode'
            />
        </div>
    );
}

export default TimecodeTag;